#!/usr/bin/perl
use strict;
use WWW::Curl::easy;
use Time::Local;

############################################################################################
#  perl_speedchex_serverlinkadvanced.pl
############################################################################################
#
# This script is a functioning example that shows perl programmers how to utilize 
# the SpeedChex ServerLink Advanced API to submit checking account and customer 
# data to SpeedChex for authorization.  The script also shows sample code for parsing
# the results sent back from SpeedChex.  Please contact your support representative
# if you do not have a copy of the SpeedChex ServerLink Advanced API documentation.
#
# In order to establish an HTTPS connection, pass data to the SpeedChex Gateway and receive
# a response, you will need to have cURL installed (http://curl.haxx.se).  After cURL is 
# installed, you will also need to install the perl binding for cURL called WWW::Curl 2.0 
# which can be # found at http://curl.haxx.se/libcurl/perl/WWW-Curl-2.0.tar.gz
#
# This script expects to receive the following data fields as POST or QUERY STRING (GET)
# variables:
#     
#     InvoiceNumber
#     CheckAmount
#     CustomerID
#     FirstName
#     LastName
#     Company
#     Address
#     City
#     State
#     Zip
#     Phone
#     Email
#     BankName
#     AccountType
#     AccountClass
#     RoutingNumber
#     CheckNumber
#     AccountNumber
#
# The data format specifications and rules about which fields are required and which fields
# are optional can be found in the SpeedChex ServerLink Advanced API documentation.
# 
# Programmers may use all or any part of this code and can make whatever modifications are
# necessary to integrate this code into their perl application.
#
# When you are ready to test this code, you will need to change the MerchantID,
# LoginName, and Password fields to those provided by your SpeedChex support representative.
#
###############################################################################################




#############################################
#  Misc Functions
#############################################

# Use this function to properly encode data for HTTP GET url string
sub URLEncode {
    my $theURL = $_[0];
   $theURL =~ s/([\W])/"%" . uc(sprintf("%2.2x",ord($1)))/eg;
   return $theURL;
}

# Use this function to format a date passed to it in the localtime() format to
# the format required by the SpeedChex Gateway
sub FormatDate {
  my @mydate = $_[1];

  my $hour = $_[2] ;
  my $minute = $_[1];
  my $second = $_[0];

  my $day = $_[3];
  my $month = ($_[4] +1);     # Month starts from 0, e.g. January is 0
  my $year = ($_[5] + 1900);  # year is expressed in years since 1900, so 4 digit representation you add 1900

  #Include leading zero if necessary
  if (length($day) == 1) {
    $day = '0' . $day
  }

  #Include leading zero if necessary
  if (length($month) == 1) {
    $month = '0' . $month
  }

  return $month . $day . $year;
}


print "Content-type: text/html\n\n";


#######################################################
# Build $REQUEST array to access POST or GET variables
#######################################################

my $buffer;
# Put POST data into $buffer scalar
if ($ENV{'REQUEST_METHOD'} eq "POST") {
  read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
} 

# Put GET (Query String) data into $buffer scalar
if ($ENV{'QUERY_STRING'}) {
  $buffer = $ENV{'QUERY_STRING'};
}

# Build $REQUEST array to access POST or GET variables
my @pairs = split(/&/, $buffer);

my %REQUEST;
foreach my $pair (@pairs)
{
  (my $name, my $value) = split(/=/, $pair);

  $value =~ tr/+/ /;
  $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $name =~ tr/+/ /;
  $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

  $REQUEST{$name} = $value;
}

#############################################
#  Build HTTPS GET string
#############################################

my $url = 'https://www.speedchex.com/datalinks/merchants/serverlink-advanced.asp' .
          '?MerchantID=2001' .
          '&GateID=test' .
          '&GateKey=test' .
          '&TestMode=on' .
          '&ReferenceID=' . &URLEncode($REQUEST{"InvoiceNumber"}) .
          '&PaymentDirection=FromCustomer' .
          '&CheckAmount=' . &URLEncode($REQUEST{"CheckAmount"}) .
          '&DateScheduled=' . &URLEncode(&FormatDate(localtime(time))) . 
          '&CustomerID=' . &URLEncode($REQUEST{"CustomerID"}) .
          '&CustomerName=' . &URLEncode($REQUEST{"FirstName"} . ' ' . $REQUEST{"LastName"}) .
          '&Company=' . &URLEncode($REQUEST{"Company"}) .
          '&Address1=' . &URLEncode($REQUEST{"Address"}) .
          '&City=' . &URLEncode($REQUEST{"City"}) .
          '&State=' . &URLEncode($REQUEST{"State"}) .
          '&Zip=' . &URLEncode($REQUEST{"Zip"}) .
          '&Phone=' . &URLEncode($REQUEST{"Phone"}) .
          '&Email=' . &URLEncode($REQUEST{"Email"}) .
          '&SendEmailToCustomer=Yes' .
          '&BankName=' . &URLEncode($REQUEST{"BankName"}) .
          '&AccountType=' . &URLEncode($REQUEST{"AccountType"}) .
          '&AccountClass=' . &URLEncode($REQUEST{"AccountClass"}) .
          '&RoutingNumber=' . &URLEncode($REQUEST{"RoutingNumber"}) .
          '&CheckNumber=' . &URLEncode($REQUEST{"CheckNumber"}) .
          '&AccountNumber=' . &URLEncode($REQUEST{"AccountNumber"});

print "<b>HTTP GET string sent to SpeedChex:</b><BR>";
print $url . "<BR><BR>";



#############################################
#  Establish cURL connection
#############################################
# Init the curl session
my $curl= WWW::Curl::easy->new() or die "curl init failed!\n";

# Follow location headers
$curl->setopt(CURLOPT_FOLLOWLOCATION, 1);

# Add some additional headers to the http-request:
my @myheaders=(
    "User-Agent: Perl interface for libcURL"
);

$curl->setopt(CURLOPT_HTTPHEADER, \@myheaders);
                                                                        
$curl->setopt(CURLOPT_URL, $url);

sub body_callback {
    my ($chunk,$context)=@_;
    push @{$context}, $chunk;
    return length($chunk); # OK
}

$curl->setopt(CURLOPT_WRITEFUNCTION, \&body_callback);

my @body;
$curl->setopt(CURLOPT_FILE, \@body);

if ($curl->perform() != 0) {
    print "Failed ::".$curl->errbuf."\n";
};

my $result;
foreach my $bodyelement (@body)
{
  $result = $result . $bodyelement;
}

#
# Cleanup is automatic
#
print "<BR><BR><BR><b>Response from SpeedChex Gateway:</b><BR>";
print $result;

#######################################################
#  Parse the response from SpeedChex Gateway
#    Note: These output messages are just examples.   
#          Modify this code to control how you want
#          to interact with your customer if there
#          is a problem.
#######################################################

print "<BR><BR><BR><b>Parsed Response:</b><BR>";
# put parsing code here
# This code will parse the result and help you respond accordingly if you choose to remove the /* and */ remarks

  # Parse results into variables.  NVPair is short for Name Value Pair
  (my $ReturnCodeNVPair, my $ErrorDescriptionNVPair, my $ErrorFieldNVPair, my $TrackingCode) = split(/,/, $result);

   if ($ReturnCodeNVPair eq 'ReturnCode=000') {
     #Put code here to conclude the customer's transaction
     print 'Authorization Successful';
   }
   else {
     #Put code here to let your customer know how they should proceed based on the specific error
     (my $ReturnCodeName, my $ReturnCodeValue) = split(/=/,$ReturnCodeNVPair);
     (my $ErrorDescriptionName, my $ErrorDescriptionValue) = split(/=/,$ErrorDescriptionNVPair);
     (my $ErrorFieldName, my $ErrorFieldValue) = split(/=/,$ErrorFieldNVPair);

     if ($ReturnCodeValue eq '001') {
       #Invalid Login
   	print 'You are not using valid login information.  This is not a customer issue.';
     }
     elsif ($ReturnCodeValue eq '002') {
	#Required Field Missing
	print 'A field required by SpeedChex (' . $ErrorFieldValue . ') has not been sent.  This is not a customer issue.';
     }
     elsif ($ReturnCodeValue eq '003') {
       #Field Value Violates Format Constraint
       #  We request that you have your program do error checking instead of using SpeedChex to do error
       #  checking for you.  If you get this error, you will want to fix your error checking process.
       #  The $ErrorFieldValue variable will show you what field has invalid data.

       print 'Invalid data contained in the ' . $ErrorFieldValue . ' field.';
     }
     elsif ($ReturnCodeValue eq '004') {
       #Field Value Exceeds Maximum Length
 	#  We request that you have your program do error checking instead of using SpeedChex to do error
 	#  checking for you.  If you get this error, you will want to fix your error checking process.
       #  The $ErrorFieldValue variable will show you what field has invalid data.

	print 'Data exceeds maximum length in the ' . $ErrorFieldValue . ' field.';
     }
     elsif ($ReturnCodeValue eq '005') {
       #Routing Number Fails Authentication
 	#  The customer has entered an invalid routing number.
					
	print 'You have entered an invalid routing number.';
     }
     elsif ($ReturnCodeValue eq '006') {
       #Transaction Request Already Processed
 	#  SpeedChex has already processed this check and will not process it again
					
	print 'This check has already been accepted and does not need to be submitted again.';
     }
     elsif ($ReturnCodeValue eq "007") {
       #Please Contact Support Representative
 	print 'This is an internal security response and is not a customer related issue.  Please contact a SpeedChex support person.';
     }
     elsif ($ReturnCodeValue eq '008') {
       #Failed Verification
 	#  The $ErrorDescriptionValue variable will contain the SpeedChex Express Verify Response Code 
	#  and Response Text separated by a colon.  See document SpeedChex_ExpressVerify_Response_Codes.doc 
	#  for a list of all possible negative response codes.
				
	#Get the SpeedChex Express Verify Response Text			
       (my $ExpressVerifyResponseCode, my $ExpressVerifyResponseText) = split(/:/,$ErrorDescriptionValue);
	print 'Account Failed Verification: &nbsp;We apologize, but we cannot accept a check on this bank account. &nbsp;The account has
		failed verification due to its current status.<BR>
		<BR>
		Your business is very important to us. &nbsp;Please review the routing number and the
		account number for any input errors. &nbsp;If you are sure the account information is correct, you
		may want to contact your banking institution to discuss the current status of your account and how
		it may have affected this transaction.';
     }
     else {
       #All other response codes
       print 'This response code (' . $ResponseCodeValue . ') indicates an unknown error and requires assistance
 		      from the SpeedChex technical support staff.'
     } 
   } # if ($ReturnCodeNVPair eq 'ReturnCode=000') else


  
